#
# Licensed Materials - Property of IBM
#
# OCO Source Materials
#
# (C) Copyright IBM Corp. 2010 All Rights Reserved
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. Copyright Office.
#
# Return codes:
#	1: usage not correct
#	2: interface is not active
#	3: input IP address is not the same as IP address retrieved from lsnetcfg
#	4: input subnet mask is not the same as subnet mask retrieved from lsnetcfg
#	5: input hostname is not the same as hostname retrieved from lsnetcfg
#	6: input domain is not the same as domain retrieved from lsnetcfg
#	7: input gateway is not the same as gateway retrieved from lsnetcfg
#	8: input dns server(s) are not the same as dns server(s) retrieved from lsnetcfg
#	9: ping of gateway failed
#	10: ping of dns server failed
#	11: ping of IP address failed
#	12: hostname or IP address retrieved from host command does not match
#          hostname or IP address retrieved from lsnetcfg command
#   13: reserved for route errors found in java interface
#	14: input IPv6 address is not the same as IPv6 address retrieved from lsnetcfg
#	15: input prefix is not the same as prefix retrieved from lsnetcfg
#	16: input prefix is not the same as prefix retrieved from lsnetcfg
#	17: reverse lookup is not working or is returning a different hostname

#!/bin/bash

#############################################################

OUTFILE=/opt/ibm/director/vmi/data/validateNetwork.out
:>$OUTFILE

function writeOutFileAndExit {
  echo RC=$1 >> $OUTFILE
  echo DHCP=$DHCP_STATUS >> $OUTFILE
  if [ $# == 2 ]; then
    echo ERROR_INFO=$2 >> $OUTFILE
  fi
  if [ "$CONTINUE_ON_ERROR" -eq 0 ]; then
    sysadminID=`cat /opt/ibm/director/lwi/conf/overrides/isdvmi.properties | grep com.ibm.director.vmi.usr.SysAdmin | sed 's/.*=//'`
    usermod -G "" $sysadminID
    userdel -r $sysadminID
    /bin/rm -r --force /home/$sysadminID
    /opt/ibm/director/vmi/bin/chnetcfg -s modify -i eth1 clear    
    sed -i '/ProductActivation class="cfgeth/,/<\/ProductActivation/ s/.*//' /opt/ibm/ae/AR/ovf-env.ar
    sed -i '/ProductActivation class="cfgdns/,/<\/ProductActivation/ s/.*//' /opt/ibm/ae/AR/ovf-env.ar
    sed -i '/ProductActivation class="cfghostgateway/,/<\/ProductActivation/ s/.*//' /opt/ibm/ae/AR/ovf-env.ar
    sed -i '/ProductActivation class="firewall/,/<\/ProductActivation/ s/.*//' /opt/ibm/ae/AR/ovf-env.ar
    rm /opt/ibm/director/vmi/data/.wizardcompleted
    rm /opt/ibm/director/vmi/data/.redirectwizard
    rm /opt/ibm/director/vmi/data/.genesiscompleted
    rm /opt/ibm/director/vmi/data/reboot.out
    cp /opt/ibm/director/vmi/data/hosts.backup /etc/hosts
    touch /opt/ibm/director/vmi/data/networkValidation.error
    /usr/smrshbin/smshutdown -r -t now && sleep 1000
  else
    rm /tmp/immAddresses.out
  fi
  exit $1
}

function getLsnetcfgValue {
  local VALUE=$(echo $LSNETCFG | awk -F"$1" '{print $2}' | awk -F"$2" '{print $1}')
  VALUE="${VALUE#"${VALUE%%[![:space:]]*}"}"   # remove leading whitespace characters
  VALUE="${VALUE%"${VALUE##*[![:space:]]}"}"   # remove trailing whitespace characters
  if [[ $VALUE == \"* ]]; then                 # remove quotes
    VALUE=${VALUE:1:${#VALUE}-2}
  fi
  echo $VALUE
}

function normalize {
  local VALUE=$1
  local VALUE_LEN=${#VALUE}
  local PAD=$(expr 4 - $VALUE_LEN)
  local loopCount=0
  while [ $loopCount -lt $PAD ]; do
    VALUE=0$VALUE
    loopCount=$(expr $loopCount + 1)
  done
  echo $VALUE
}

function expand {
  local VALUE=$1
  local COLONS_TO_ADD="::"
  local EXPANDED=
  local NORMALIZED=
  if [ -n "$(echo $VALUE | grep "::")" ]; then
    local FRONT=$(echo $VALUE | awk -F"::" '{print $1}')
    local BACK=$(echo $VALUE | awk -F"::" '{print $2}')
    IFS=":"
    local COLON_COUNT=0
    for i in $VALUE; do
      COLON_COUNT=$(expr $COLON_COUNT + 1)
    done
    unset IFS    
    local PAD=$(expr 8 - $COLON_COUNT)
    local loopCount=0
    while [ $loopCount -lt $PAD ]; do
      COLONS_TO_ADD=\:$COLONS_TO_ADD
      loopCount=$(expr $loopCount + 1)
    done
    EXPANDED=$FRONT$COLONS_TO_ADD$BACK
  else
    EXPANDED=$VALUE
  fi
  IFS=":"
  for i in $EXPANDED; do
    NORMALIZED=$NORMALIZED\:$(normalize $i)
  done
  unset IFS    
  if [[ $NORMALIZED == \:* ]]; then                 # remove colon
    NORMALIZED=${NORMALIZED:1:${#NORMALIZED}-1}
  fi
  echo $NORMALIZED
}

#
# Get input parms
#
INTERFACE=
INPUT_IP_ADDR=
INPUT_IPV6_ADDR=
INPUT_SUBNET_MASK=
INPUT_HOSTNAME=
INPUT_DOMAIN=
INPUT_DNS_SERVERS=
INPUT_GATEWAY=
INPUT_RETURN_CODE=
INPUT_PREFIX=
INPUT_HOSTNAME_IP_ADDR=
CONTINUE_ON_ERROR=0
while getopts "i:a:b:s:h:d:n:g:r:p:q:c" OPTION
do
  case $OPTION in
  i)  INTERFACE="$OPTARG"
      ;;
  a)  INPUT_IP_ADDR="$OPTARG"
      ;;
  b)  INPUT_IPV6_ADDR="$OPTARG"
      ;;
  s)  INPUT_SUBNET_MASK="$OPTARG"
      ;;
  h)  INPUT_HOSTNAME="$OPTARG"
      ;;
  d)  INPUT_DOMAIN="$OPTARG"
      ;;
  n)  INPUT_DNS_SERVERS="$OPTARG"
      ;;
  g)  INPUT_GATEWAY="$OPTARG"
      ;;
  r)  INPUT_RETURN_CODE="$OPTARG"
      ;;
  p)  INPUT_PREFIX="$OPTARG"
      ;;
  q)  INPUT_HOSTNAME_IP_ADDR="$OPTARG"
      ;;
  c)  CONTINUE_ON_ERROR=1
      ;;
  ?)  printf "Usage: %s: -i "eth0"|"eth1" [-a ip address] [-b ipv6 address] [-s subnet mask] [-p prefix] [-h hostname] [-d domain] [-n dns servers] [-g gateway] [-r return code] [-q hostname ip address] [-c]\n" $(basename $0) >&2
      writeOutFileAndExit 1 
      ;;
  esac
  echo -n - >> $OUTFILE
  if [ "$OPTION" = "c" ]; then
    echo $OPTION >> $OUTFILE
  else
    echo $OPTION $OPTARG >> $OUTFILE
  fi
done
if [[ -z "$INTERFACE" || "$INTERFACE" != [Ee][Tt][Hh]0 && "$INTERFACE" != [Ee][Tt][Hh]1 ]]; then
  printf "Usage: %s: -i "eth0"|"eth1" [-a ip address] [-b ipv6 address] [-s subnet mask] [-p prefix] [-h hostname] [-d domain] [-n dns servers] [-g gateway] [-r return code] [-q hostname ip address] [-c]\n" $(basename $0) >&2
  writeOutFileAndExit 1
fi

#
# get IP address, IPV6 address, netmask, gateway, etc...
#
LSNETCFG=$(/opt/ibm/director/vmi/bin/lsnetcfg -n)
HOSTNAME=$(getLsnetcfgValue hostname: domain:)
DOMAIN=$(getLsnetcfgValue domain: ipaddr:)
IP_ADDR=$(getLsnetcfgValue ipaddr: networkmask:)
IPV6_ADDR=$(getLsnetcfgValue ipv6addr: gateway:)
NETMASK=$(getLsnetcfgValue networkmask: ipv6addr:)
GATEWAY=$(getLsnetcfgValue gateway: nameserver:)
NAME_SERVER=$(getLsnetcfgValue nameserver: domainsuffix:)
DOMAIN_SUFFIX=$(getLsnetcfgValue domainsuffix: ntpstatus:)
NTP_STATUS=$(getLsnetcfgValue ntpstatus: dhcpstatus:)
DHCP_STATUS=$(getLsnetcfgValue dhcpstatus: ntpserver:)
DHCP_ETH0_STATUS=$(getLsnetcfgValue ipv4dhcp_eth0: ipv6addr_eth0:)
DHCP_ETH1_STATUS=$(getLsnetcfgValue ipv4dhcp_eth1: ipv6addr_eth1:)
DHCP_ETH0_IPV6_STATUS=$(getLsnetcfgValue ipv6dhcp_eth0: dhcpserver_eth0:)
DHCP_ETH1_IPV6_STATUS=$(getLsnetcfgValue ipv6dhcp_eth1: dhcpserver_eth1:)
NTP_SERVER=$(getLsnetcfgValue ntpserver: syslogserver:)
if [[ "$INTERFACE" == [Ee][Tt][Hh]0 ]]; then
  IP_ADDR=$(getLsnetcfgValue ipv4addr_eth0: ipv4netmask_eth0:)
  IPV6_ADDR=$(getLsnetcfgValue ipv6addr_eth0: ipv6auto_eth0:)
  NETMASK=$(getLsnetcfgValue ipv4netmask_eth0: mac_address_eth0)
elif [[ "$INTERFACE" == [Ee][Tt][Hh]1 ]]; then
  IP_ADDR=$(getLsnetcfgValue ipv4addr_eth1: ipv4netmask_eth1:)
  IPV6_ADDR=$(getLsnetcfgValue ipv6addr_eth1: ipv6auto_eth1:)
  NETMASK=$(getLsnetcfgValue ipv4netmask_eth1: mac_address_eth1)
fi
if [[ -n "$IPV6_ADDR" && $(expr index "$IPV6_ADDR" /) -gt 0 ]]; then
  PREFIX=${IPV6_ADDR:$(expr index "$IPV6_ADDR" /)}
  IPV6_ADDR=${IPV6_ADDR:0:$(expr $(expr index "$IPV6_ADDR" /) - 1)}
fi

#
# Check if return code passed in. If so, do exit processing
# with specified return code
#
if [[ -n "$INPUT_RETURN_CODE" && "$INPUT_RETURN_CODE" == [0123456789] || "$INPUT_RETURN_CODE" == [0123456789][0123456789] ]]; then
  writeOutFileAndExit $INPUT_RETURN_CODE
fi

#
# Check if interface is active
#
if [ -z "$(ifconfig $INTERFACE | grep "UP")" ]; then
  writeOutFileAndExit 2
fi

#
# validate that values specified in setup wizard are values being used
# in current network configuration
#
if [[ -n "$INPUT_IP_ADDR" && "$INPUT_IP_ADDR" != "$IP_ADDR" ]]; then
  writeOutFileAndExit 3 $IP_ADDR
fi

#if [[ -n "$INPUT_SUBNET_MASK" && "$INPUT_SUBNET_MASK" != "$NETMASK" ]]; then
#  writeOutFileAndExit 4 $NETMASK
#fi

#if [[ -n "$INPUT_DNS_SERVERS" ]]; then
#  IFS=","
#  count1=0
#  for i in $NAME_SERVER
#  do
#    count1=$(expr $count1 + 1)
#  done
#  count2=0
#  for i in $INPUT_DNS_SERVERS
#  do
#    count2=$(expr $count2 + 1)
#  done
#  if [[ "$count1" -ne "$count2" ]]; then
#    writeOutFileAndExit 8 $NAME_SERVER
#  else
#    for i in $NAME_SERVER
#    do
#      found=0
#      for j in $INPUT_DNS_SERVERS
#      do
#        if [[ "$i" == "$j" ]]; then
#          found=1
#        fi
#      done
#      if [[ "$found" -eq 0 ]]; then
#        writeOutFileAndExit 8 $i
#      fi
#    done
#  fi
#fi

#
# ping gateway
#
if [[ -n "$INPUT_GATEWAY" ]]; then
  if [ -z "$(echo $INPUT_GATEWAY | grep "0.0.0.0")" ]; then
    arping -f -I $INTERFACE $INPUT_GATEWAY -w 5 &>/dev/null
    rc=$?
    if [ "$rc" -ne 0 ]; then
      writeOutFileAndExit 9 $GATEWAY
    fi
  fi
fi

#
# ping DNS server(s)
#
IFS=","
if [[ -n "$INPUT_DNS_SERVERS" ]]; then
for i in $INPUT_DNS_SERVERS
do
  if [ -n "$(host $i | grep "no servers could be reached")" ]; then
    writeOutFileAndExit 10 $i
  fi
done
fi
unset IFS

#
# ping hostname and validate with host command
#
if [[ -n "$INPUT_HOSTNAME_IP_ADDR" ]]; then
	if [ -n "$(echo $INPUT_HOSTNAME_IP_ADDR | grep ":")" ]; then
		ping6 -c 1 -w 5 $HOSTNAME &>/dev/null
	else
		ping -c 1 -w 5 $HOSTNAME &>/dev/null
	fi
	rc=$?
	if [ $rc -ne 0 ]; then
  		writeOutFileAndExit 12 $HOSTNAME
	fi
fi
if [[ -n "$INPUT_HOSTNAME_IP_ADDR" ]]; then
  if [[ -n "$INPUT_DNS_SERVERS" ]]; then
    if [ "$DHCP_ETH0_STATUS" == "off" -a "$DHCP_ETH1_STATUS" == "off" -a "$DHCP_ETH0_IPV6_STATUS" == "off" -a "$DHCP_ETH1_IPV6_STATUS" == "off" ]; then
    if [[ "$INPUT_HOSTNAME_IP_ADDR" == "$IP_ADDR" ]]; then
    HOST_IP_ADDR=$(host $HOSTNAME | awk -F"has address " '{print $2}')
    if [[ "$HOST_IP_ADDR" != "$IP_ADDR" ]]; then
      writeOutFileAndExit 12 $HOST_IP_ADDR
    fi
    HOST_HOSTNAME=$(host $IP_ADDR | awk -F"domain name pointer " '{print $2}' | awk -F"." '{print $1}')
    
# Temporarily turn off case sensitivity on string comparison.  Reverse lookup can return mixed case hostnames and
# cause network validation failures that are not real failures.    
    shopt -s nocasematch
  
    if [[ "$HOST_HOSTNAME" != "$HOSTNAME" ]]; then
      writeOutFileAndExit 17 $IP_ADDR
    fi
    
# Turn case sensitivity back on for string comparisons
   shopt -u nocasematch   
    
    fi
#  else
#    NORMALIZED_INPUT_HOSTNAME_IP_ADDR=$(expand $INPUT_HOSTNAME_IP_ADDR)
#    NORMALIZED_IPV6_ADDR=$(expand $IPV6_ADDR)
#    if [[ "$NORMALIZED_INPUT_HOSTNAME_IP_ADDR" == "$NORMALIZED_IPV6_ADDR" ]]; then
#      HOST_IP_ADDR=$(host $HOSTNAME | awk -F"has address " '{print $2}')
#      NORMALIZED_HOST_IP_ADDR=$(expand $HOST_IP_ADDR)
#      if [[ "$NORMALIZED_HOST_IP_ADDR" != "$NORMALIZED_IPV6_ADDR" ]]; then
#        writeOutFileAndExit 12 $HOST_IP_ADDR
#      fi
#      HOST_HOSTNAME=$(host $IPV6_ADDR | awk -F"domain name pointer " '{print $2}' | awk -F"." '{print $1}')
#      if [[ "$HOST_HOSTNAME" != "$HOSTNAME" ]]; then
#        writeOutFileAndExit 12 $HOST_HOSTNAME
#      fi
#    fi
  fi
  fi
fi

IP_AD_ETH0=$(getLsnetcfgValue ipv4addr_eth0: ipv4netmask_eth0:)
IP_AD_ETH1=$(getLsnetcfgValue ipv4addr_eth1: ipv4netmask_eth1:)
NETMASK_ETH0=$(getLsnetcfgValue ipv4netmask_eth0: mac_address_eth0)
NETMASK_ETH1=$(getLsnetcfgValue ipv4netmask_eth1: mac_address_eth1)
if [ "$IP_AD_ETH0" != "0.0.0.0" -a "$IP_AD_ETH1" != "0.0.0.0" -a "$IP_AD_ETH0" != "" -a "$IP_AD_ETH1" != "" ]; then
	OCT1_ETH0=$(echo $IP_AD_ETH0 | cut -d '.' -f1)
	OCT1_MASK0=$(echo $NETMASK_ETH0 | cut -d '.' -f1)
	OCT1_AND0=$(( $OCT1_ETH0 & $OCT1_MASK0 ))
	OCT2_ETH0=$(echo $IP_AD_ETH0 | cut -d '.' -f2)
	OCT2_MASK0=$(echo $NETMASK_ETH0 | cut -d '.' -f2)
	OCT2_AND0=$(( $OCT2_ETH0 & $OCT2_MASK0 ))
	OCT3_ETH0=$(echo $IP_AD_ETH0 | cut -d '.' -f3)
	OCT3_MASK0=$(echo $NETMASK_ETH0 | cut -d '.' -f3)
	OCT3_AND0=$(( $OCT3_ETH0 & $OCT3_MASK0 ))
	OCT4_ETH0=$(echo $IP_AD_ETH0 | cut -d '.' -f4)
	OCT4_MASK0=$(echo $NETMASK_ETH0 | cut -d '.' -f4)
	OCT4_AND0=$(( $OCT4_ETH0 & $OCT4_MASK0 ))
	
	OCT1_ETH1=$(echo $IP_AD_ETH1 | cut -d '.' -f1)
	OCT1_MASK1=$(echo $NETMASK_ETH1 | cut -d '.' -f1)
	OCT1_AND1=$(( $OCT1_ETH1 & $OCT1_MASK1 ))
	OCT2_ETH1=$(echo $IP_AD_ETH1 | cut -d '.' -f2)
	OCT2_MASK1=$(echo $NETMASK_ETH1 | cut -d '.' -f2)
	OCT2_AND1=$(( $OCT2_ETH1 & $OCT2_MASK1 ))
	OCT3_ETH1=$(echo $IP_AD_ETH1 | cut -d '.' -f3)
	OCT3_MASK1=$(echo $NETMASK_ETH1 | cut -d '.' -f3)
	OCT3_AND1=$(( $OCT3_ETH1 & $OCT3_MASK1 ))
	OCT4_ETH1=$(echo $IP_AD_ETH1 | cut -d '.' -f4)
	OCT4_MASK1=$(echo $NETMASK_ETH1 | cut -d '.' -f4)
	OCT4_AND1=$(( $OCT4_ETH1 & $OCT4_MASK1 ))
		
	if [ "$OCT1_AND0" == "$OCT1_AND1" -a "$OCT2_AND0" == "$OCT2_AND1" -a "$OCT3_AND0" == "$OCT3_AND1" -a "$OCT4_AND0" == "$OCT4_AND1" ]; then
		errorSTR=$IP_AD_ETH0','$IP_AD_ETH1
		writeOutFileAndExit 16 $errorSTR		
	fi

fi

IPV6_ETH0=$(getLsnetcfgValue ipv6addr_eth0: ipv6auto_eth0:)
IPV6_ETH1=$(getLsnetcfgValue ipv6addr_eth1: ipv6auto_eth1:)
if [[ -n "$IPV6_ETH0" && $(expr index "$IPV6_ETH0" /) -gt 0 ]]; then
	arr=$(echo $IPV6_ETH0 | tr "," "\n")
	for x in $arr
	do
		IPV6_ETH0_PREFIX=${x:$(expr index "$x" /)}
		IPV6_ETH0_ADDR=${x:0:$(expr $(expr index "$x" /) - 1)}
		if [ -n "$(echo $IPV6_ETH0_ADDR | grep 'fe80')" ]; then
			continue
		fi
		if [ -z "$(echo $IPV6_ETH0_PREFIX | grep '64')" ]; then
			continue
		fi 
		IPV6_ETH0_ADDR=$(expand $IPV6_ETH0_ADDR)
		IPV6_ETH0_ADDR_SUB=`echo $IPV6_ETH0_ADDR | awk -F":" '{print $1":"$2":"$3":"$4}'`
		
		if [[ -n "$IPV6_ETH1" && $(expr index "$IPV6_ETH1" /) -gt 0 ]]; then
	    	arr=$(echo $IPV6_ETH1 | tr "," "\n")
			for y in $arr
			do
				IPV6_ETH1_PREFIX=${y:$(expr index "$y" /)}
				IPV6_ETH1_ADDR=${y:0:$(expr $(expr index "$y" /) - 1)}
				if [ -n "$(echo $IPV6_ETH1_ADDR | grep 'fe80')" ]; then
					continue
				fi
				if [ -z "$(echo $IPV6_ETH1_PREFIX | grep '64')" ]; then
					continue
				fi 
				IPV6_ETH1_ADDR=$(expand $IPV6_ETH1_ADDR)
				IPV6_ETH1_ADDR_SUB=`echo $IPV6_ETH1_ADDR | awk -F":" '{print $1":"$2":"$3":"$4}'`			
				if [ "$IPV6_ETH0_ADDR_SUB" == "$IPV6_ETH1_ADDR_SUB" ]; then
					echo 'stateless was same subnet, disabling stateless autoconfig for eth1. Conflicting addresses are: '$IPV6_ETH0_ADDR','$IPV6_ETH1_ADDR >> $OUTFILE
    				/opt/ibm/director/vmi/bin/chnetcfg -s modify -i eth1 --ipv6auto off
    				ifdown eth1
    				ifup eth1					
				fi
			done
		fi
	done
fi
rm /tmp/immAddresses.out
rm /opt/ibm/director/vmi/data/networkValidation.error
exit 0